import * as Reactivity from "@effect/experimental/Reactivity";
import * as Client from "@effect/sql/SqlClient";
import { SqlError } from "@effect/sql/SqlError";
import * as Statement from "@effect/sql/Statement";
import * as Chunk from "effect/Chunk";
import * as Config from "effect/Config";
import * as Context from "effect/Context";
import * as Effect from "effect/Effect";
import { identity } from "effect/Function";
import * as Layer from "effect/Layer";
import * as Scope from "effect/Scope";
import * as Stream from "effect/Stream";
/**
 * @category type ids
 * @since 1.0.0
 */
export const TypeId = /*#__PURE__*/Symbol.for("@effect/sql-sqlite-do/SqliteClient");
/**
 * @category tags
 * @since 1.0.0
 */
export const SqliteClient = /*#__PURE__*/Context.GenericTag("@effect/sql-sqlite-do/SqliteClient");
/**
 * @category constructor
 * @since 1.0.0
 */
export const make = options => Effect.gen(function* () {
  const compiler = Statement.makeCompilerSqlite(options.transformQueryNames);
  const transformRows = options.transformResultNames ? Statement.defaultTransforms(options.transformResultNames).array : undefined;
  const makeConnection = Effect.gen(function* () {
    const db = options.db;
    function* runIterator(sql, params = []) {
      const cursor = db.exec(sql, ...params);
      const columns = cursor.columnNames;
      for (const result of cursor.raw()) {
        const obj = {};
        for (let i = 0; i < columns.length; i++) {
          const value = result[i];
          obj[columns[i]] = value instanceof ArrayBuffer ? new Uint8Array(value) : value;
        }
        yield obj;
      }
    }
    const runStatement = (sql, params = []) => Effect.try({
      try: () => Array.from(runIterator(sql, params)),
      catch: cause => new SqlError({
        cause,
        message: `Failed to execute statement`
      })
    });
    const runValues = (sql, params = []) => Effect.try({
      try: () => Array.from(db.exec(sql, ...params).raw(), row => {
        for (let i = 0; i < row.length; i++) {
          const value = row[i];
          if (value instanceof ArrayBuffer) {
            row[i] = new Uint8Array(value);
          }
        }
        return row;
      }),
      catch: cause => new SqlError({
        cause,
        message: `Failed to execute statement`
      })
    });
    return identity({
      execute(sql, params, transformRows) {
        return transformRows ? Effect.map(runStatement(sql, params), transformRows) : runStatement(sql, params);
      },
      executeRaw(sql, params) {
        return runStatement(sql, params);
      },
      executeValues(sql, params) {
        return runValues(sql, params);
      },
      executeUnprepared(sql, params, transformRows) {
        return transformRows ? Effect.map(runStatement(sql, params), transformRows) : runStatement(sql, params);
      },
      executeStream(sql, params, transformRows) {
        return Stream.suspend(() => {
          const iterator = runIterator(sql, params);
          return Stream.fromIteratorSucceed(iterator, 16);
        }).pipe(transformRows ? Stream.mapChunks(chunk => Chunk.unsafeFromArray(transformRows(Chunk.toReadonlyArray(chunk)))) : identity);
      }
    });
  });
  const semaphore = yield* Effect.makeSemaphore(1);
  const connection = yield* makeConnection;
  const acquirer = semaphore.withPermits(1)(Effect.succeed(connection));
  const transactionAcquirer = Effect.uninterruptibleMask(restore => Effect.as(Effect.zipRight(restore(semaphore.take(1)), Effect.tap(Effect.scope, scope => Scope.addFinalizer(scope, semaphore.release(1)))), connection));
  return Object.assign(yield* Client.make({
    acquirer,
    compiler,
    transactionAcquirer,
    spanAttributes: [...(options.spanAttributes ? Object.entries(options.spanAttributes) : []), ["db.system", "sqlite"]],
    transformRows
  }), {
    [TypeId]: TypeId,
    config: options
  });
});
/**
 * @category layers
 * @since 1.0.0
 */
export const layerConfig = config => Layer.scopedContext(Config.unwrap(config).pipe(Effect.flatMap(make), Effect.map(client => Context.make(SqliteClient, client).pipe(Context.add(Client.SqlClient, client))))).pipe(Layer.provide(Reactivity.layer));
/**
 * @category layers
 * @since 1.0.0
 */
export const layer = config => Layer.scopedContext(Effect.map(make(config), client => Context.make(SqliteClient, client).pipe(Context.add(Client.SqlClient, client)))).pipe(Layer.provide(Reactivity.layer));
//# sourceMappingURL=SqliteClient.js.map